! =================================================================================================================================
! MODULE       : sapiens_kill
!
! CONTACT      : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       Kill all vegetation scheduled for killing by human activities.
!!
!!\n DESCRIPTION : Actually kills biomass.  The biomass is scheduled for killing
!!       by the variable circ_class_kill in other sapiens routines.  Here the
!!       biomass is moved to the litter and harvest pools, as appropriate.
!!
!! RECENT CHANGE(S): None
!!
!! REFERENCE(S) : 
!! - Sitch, S., B. Smith, et al. (2003), Evaluation of ecosystem dynamics,
!!         plant geography and terrestrial carbon cycling in the LPJ dynamic 
!!         global vegetation model, Global Change Biology, 9, 161-185.\n
!! - Waring, R. H. (1983). "Estimating forest growth and efficiency in relation 
!!         to canopy leaf area." Advances in Ecological Research 13: 327-354.\n
!! 
!! SVN          :
!! $HeadURL: svn://forge.ipsl.jussieu.fr/orchidee/branches/ORCHIDEE-DOFOCO/ORCHIDEE/src_stomate/lpj_gap.f90 $
!! $Date: 2013-08-19 17:50:44 +0200 (Mon, 19 Aug 2013) $
!! $Revision: 1425 $
!! \n
!_ ================================================================================================================================

MODULE sapiens_kill

  ! modules used:

  USE stomate_data
  USE pft_parameters
  USE ioipsl_para 
  USE function_library,       ONLY : distribute_mortality_biomass,&
                                     nmax,  wood_to_dia
  USE constantes

  IMPLICIT NONE

  ! private & public routines

  PRIVATE
  PUBLIC gap_prog_sapiens
  
  ! Variable declaration 

CONTAINS


!! ================================================================================================================================
!! SUBROUTINE   : gap_prog_sapiens
!!
!>\BRIEF        Transfer dead biomass to litter and update stand density as a result
!!              of human activities.
!!
!! DESCRIPTION  : This routine has a simple purpose: kill individuals by human activities.  
!!   The variable circ_class_kill indicates how many individuals need to be killed by the various
!!   processes in the code, and is calculated in other modules.  gap_prog_sapiens takes
!!   this variable and decides on the correct order of which to actually kill the
!!   trees.  As these are all human activities, some of the biomass may be moved to
!!   harvest pools, while other parts are moved to litter.
!!
!! RECENT CHANGE(S): 
!!
!! MAIN OUTPUT VARIABLE(S): ::circ_class_biomass; ::biomass; ::ind
!!
!! REFERENCE(S)   :
!! - Sitch, S., B. Smith, et al. (2003), Evaluation of ecosystem dynamics,
!!         plant geography and terrestrial carbon cycling in the LPJ dynamic 
!!         global vegetation model, Global Change Biology, 9, 161-185.
!! - Waring, R. H. (1983). "Estimating forest growth and efficiency in relation 
!!         to canopy leaf area." Advances in Ecological Research 13: 327-354.
!!
!! FLOWCHART    : None
!!\n
!_ ================================================================================================================================
 
  SUBROUTINE gap_prog_sapiens (npts, biomass, ind,  &
       bm_to_litter, circ_class_biomass, circ_class_kill, &
       circ_class_n, harvest_pool_bound, harvest_pool, harvest_type, &
       harvest_cut, harvest_area, resolution, &
       veget_max, age_stand, last_cut, mai_count, circ_class_dist, &
       coppice_dens)

    !! 0. Variable and parameter declaration

    !! 0.1 Input variables 
    INTEGER(i_std), INTENT(in)                                   :: npts               !! Domain size (-)
    REAL(r_std), DIMENSION(0:ndia_harvest+1), INTENT(in)         :: harvest_pool_bound !! The boundaries of the diameter classes

                                                                                       !! in the wood harvest pools
                                                                                       !! @tex $(m)$ @endtex
    REAL(r_std),DIMENSION(npts,2), INTENT(in)                    :: resolution         !! The length of each grid-box in X and Y
                                                                                       !! @tex $(m)$ @endtex
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)                   :: veget_max          !! Maximum fraction of vegetation type 
    REAL(r_std), DIMENSION(ncirc), INTENT(in)                    :: circ_class_dist    !! The probability distribution of trees 
                                                                                       !! in a circ class in case of a 
                                                                                       !! redistribution (unitless).
    REAL(r_std), DIMENSION(npts,nvm),INTENT(in)                  :: coppice_dens       !! The density of a coppice at the first
                                                                                       !! cutting.
                                                                                       !! @tex $( 1 / m**2 )$ @endtex 

    !! 0.2 Output variables

    !! 0.3 Modified variables   
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                             INTENT(inout)       :: biomass            !! Biomass @tex $(gC m^{-2}) $@endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)              :: ind                !! Stand level number of individuals 
                                                                                       !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nparts,nelements), &                         
                                              INTENT(inout)      :: circ_class_biomass !! Biomass of the components of the model  
                                                                                       !! tree within a circumference
                                                                                       !! class @tex $(gC ind^{-1})$ @endtex  
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(inout)        :: circ_class_n       !! Number of individuals in each circ class
                                                                                       !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nfm_types,ncut_times),&
                                                  INTENT(inout)  :: circ_class_kill    !! Number of trees within a circ that needs
                                                                                       !! to be killed @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                                  INTENT(inout)  :: bm_to_litter       !! Biomass transfer to litter 
                                                                                       !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)               :: harvest_pool       !! The wood and biomass that have been
                                                                                       !! havested by humans @tex $(gC)$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)                   :: harvest_type       !! Type of management that resulted
                                                                                       !! in the harvest (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)                   :: harvest_cut        !! Type of cutting that was used for the harvest
                                                                                       !! (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)                   :: harvest_area       !! Harvested area (m^{2})
    INTEGER(i_std), DIMENSION(npts,nvm),INTENT(inout)            :: last_cut           !! Years since last thinning or clearcut (years)
    INTEGER(i_std), DIMENSION(npts,nvm),INTENT(inout)            :: age_stand          !! Age of stand (years)
    INTEGER(i_std), DIMENSION(npts,nvm),INTENT(inout)            :: mai_count          !! The number of times we've
                                                                                       !! calculated the volume increment
                                                                                       !! for a stand
 
    !! 0.4 Local variables
    INTEGER(i_std)                                               :: ipts,ivm           !! Indices
    INTEGER(i_std)                                               :: iele               !! Indices
    INTEGER(i_std)                                               :: ipar, imbc         !! Indices
    
    REAL(r_std), DIMENSION(npts,nvm,nmbcomp,nelements)           :: check_intern       !! Contains the components of the internal
                                                                                       !! mass balance chech for this routine
                                                                                       !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)                   :: closure_intern     !! Check closure of internal mass balance
                                                                                       !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)                   :: pool_start         !! Start pool of this routine 
                                                                                       !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)                   :: pool_end           !! End pool of this routine 
                                                                                       !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
!_ ================================================================================================================================

    IF (bavard.GE.2) WRITE(numout,*) 'Entering gap_prog_sapiens'

!! 2. Initialize variables

    ! 2.1 Initialize check for mass balance closure
    !  The mass balance is calculated at the end of this routine
    !  in section .
    pool_start = zero
    DO ipar = 1,nparts
       DO iele = 1,nelements
          !  Initial litter and biomass
          pool_start(:,:,iele) = pool_start(:,:,iele) + &
               (biomass(:,:,ipar,iele) * veget_max(:,:)) + &
               (bm_to_litter(:,:,ipar,iele) * veget_max(:,:))
       ENDDO
    ENDDO

    ! The biomass harvest pool shouldn't be multiplied by veget_max
    ! its units are already in gC pixel-1
    ! Right now the harvest pool is in gC.  We will divide by the
    ! resolution to make it per pixel.
    ! 
    DO ipts = 1, npts
       pool_start(ipts,:,icarbon) = pool_start(ipts,:,icarbon) + &
         SUM(harvest_pool(ipts,:,:,icarbon),2) / &
         (resolution(ipts,1) * resolution(ipts,2))
    ENDDO

 !! 3. Kill plants

    DO  ipts = 1,npts ! loop over land_points

       DO ivm = 2,nvm  ! loop over #PFT

          IF(veget_max(ipts,ivm) == zero)THEN
              ! this vegetation type is not present, so no reason to do the 
              ! calculation
              CYCLE
          ENDIF


          DO iele = 1,nelements

             ! Clearcuts and thinning can only happen on trees.
             IF(is_tree(ivm))THEN


                ! Now see if there are any plants to be killed by a clearcut.
                ! Only the stems are harvested.  This should be done before
                ! a thinning so that we don't overcount.
                CALL clearcut_harvest_aboveground_dead_roots(npts, ipts, ivm, iele, &
                     ifm_thin, icut_clear, biomass, ind, bm_to_litter, &
                     circ_class_biomass, circ_class_kill, circ_class_n, harvest_pool, &
                     harvest_type, harvest_cut, harvest_area, &
                     harvest_pool_bound, resolution, veget_max, last_cut, age_stand, &
                     mai_count, un - branch_ratio(ivm))


                ! Not only do we harvest stems during thinning, we have to adjust
                ! the mortality.  We'll do all this in a subroutine.
                CALL thinning_harvest_aboveground_dead_roots(npts, ipts, ivm, iele, &
                     ifm_thin, icut_thin, biomass, ind, bm_to_litter, &
                     circ_class_biomass, circ_class_kill, circ_class_n, harvest_pool, &
                     harvest_type, harvest_cut, harvest_area, &
                     harvest_pool_bound, resolution, veget_max, last_cut, &
                     un - branch_ratio(ivm))

                ! And the final cut of an SRC, where the underground wood dies.  This is
                ! exactly the same as the clearcut above, except that we take all
                ! of the aboveground biomass.
                CALL clearcut_harvest_aboveground_dead_roots(npts, ipts, ivm, iele, &
                     ifm_src, icut_cop3, biomass, ind, bm_to_litter, &
                     circ_class_biomass, circ_class_kill, circ_class_n, harvest_pool, &
                     harvest_type, harvest_cut, harvest_area, &
                     harvest_pool_bound, resolution, veget_max, last_cut, age_stand, &
                     mai_count, un)

               ! Have we cut a coppice system for the first time?
                CALL first_coppice_cut(npts, ipts, ivm, iele, ifm_cop, &
                     icut_cop1, harvest_pool_bound, resolution, veget_max, biomass, &
                     ind, circ_class_biomass, circ_class_n, circ_class_kill, &
                     bm_to_litter, harvest_pool, harvest_type, harvest_cut, &
                     harvest_area, last_cut)
                CALL first_coppice_cut(npts, ipts, ivm, iele, ifm_src, &
                     icut_cop1, harvest_pool_bound, resolution, veget_max, biomass, &
                     ind, circ_class_biomass, circ_class_n, circ_class_kill, &
                     bm_to_litter, harvest_pool, harvest_type, harvest_cut, &
                     harvest_area, last_cut)

                ! What if we coppice for the second (and subsequent) cuts?
                CALL second_coppice_cut(npts, ipts, ivm, iele, ifm_cop, &
                     icut_cop2, harvest_pool_bound, resolution, veget_max, biomass, &
                     ind, circ_class_biomass, circ_class_n, circ_class_kill, &
                     bm_to_litter, harvest_pool, harvest_type, harvest_cut, &
                     harvest_area, circ_class_dist, last_cut, coppice_dens)
                ! For short rotation coppices
                CALL second_coppice_cut(npts, ipts, ivm, iele, ifm_src, &
                     icut_cop2, harvest_pool_bound, resolution, veget_max, biomass, &
                     ind, circ_class_biomass, circ_class_n, circ_class_kill, &
                     bm_to_litter, harvest_pool, harvest_type, harvest_cut, &
                     harvest_area, circ_class_dist, last_cut, coppice_dens)

             ENDIF ! is_tree(ivm)

             ! Here some circ classes may be empty.  We do not want to do
             ! anything about that yet, since we have not killed the natural
             ! mortality biomass.  If we redistribute the biomass now we will
             ! change the circ_class_biomass, but the old values for this
             ! were used to determine circ_class_kill(inatural) in 
             ! stomate_mark_kill.  If we change those values we will kill 
             ! the wrong amount of biomass in lpj_gap.

          ENDDO ! loop over elements

       ENDDO  ! loop over pfts

    END DO ! loop over land points

 !! 4. Check mass balance closure
   
    !  Consider this whole routine as a black box with incoming and outgoing 
    !  fluxes and a change in the mass of the box. Express in absolute 
    !  units gC (or gN), hence, multiply with dt and veget_max. In most routines 
    !  veget_max does not change and could be omitted but a general approach 
    !  was prefered.

    !! 4.1 Calculate pools at the end of the routine
    pool_end = zero
    DO ipar = 1,nparts
       DO iele = 1,nelements
          pool_end(:,:,iele) = pool_end(:,:,iele) + &
               (biomass(:,:,ipar,iele) * veget_max(:,:)) + &
               (bm_to_litter(:,:,ipar,iele) * veget_max(:,:))
       ENDDO
    ENDDO
   
    ! The biomass harvest pool shouldn't be multiplied by veget_max
    ! its units are already in gC pixel-1
    ! Right now the harvest pool is in gC.  We will divide by the
    ! resolution to make it per pixel.
    ! 
    DO ipts = 1, npts
       pool_end(ipts,:,icarbon) = pool_end(ipts,:,icarbon) + &
            SUM(harvest_pool(ipts,:,:,icarbon),2) / &
            (resolution(ipts,1) * resolution(ipts,2))
    ENDDO

    !! 4.2 Calculate components of the mass balance
    check_intern(:,:,iatm2land,icarbon) = zero
    check_intern(:,:,iland2atm,icarbon) = -un * zero
    check_intern(:,:,ilat2out,icarbon) = zero
    check_intern(:,:,ilat2in,icarbon) = -un * zero
    check_intern(:,:,ipoolchange,icarbon) = -un * (pool_end(:,:,icarbon) - &
         pool_start(:,:,icarbon))
    closure_intern = zero
    DO imbc = 1,nmbcomp
       closure_intern(:,:,icarbon) = closure_intern(:,:,icarbon) + &
            check_intern(:,:,imbc,icarbon)
    ENDDO
 
    ! Write outcome
    DO ipts=1,npts
       DO ivm=1,nvm
          IF(ABS(closure_intern(ipts,ivm,icarbon)) .LE. min_stomate)THEN
             IF (ld_massbal) WRITE(numout,*) 'Mass balance closure in sapiens_kill.f90'
          ELSE
             WRITE(numout,*) 'Error: mass balance is not closed in sapiens_kill.f90'
             WRITE(numout,*) '   ipts,ivm; ', ipts,ivm
             WRITE(numout,*) '   Difference is, ', closure_intern(ipts,ivm,icarbon)
             WRITE(numout,*) '   pool_end,pool_start: ', pool_end(ipts,ivm,icarbon), pool_start(ipts,ivm,icarbon)
             IF(ld_stop)THEN
                CALL ipslerr_p (3,'gap_prog_sapiens', 'Mass balance error.','','')
             ENDIF
          ENDIF
       ENDDO
    ENDDO

    IF (bavard.GE.2) WRITE(numout,*) 'Leaving gap_prog_sapiens'

  END SUBROUTINE gap_prog_sapiens


!! ================================================================================================================================
!! SUBROUTINE   : thinning_harvest_aboveground_dead_roots
!!
!>\BRIEF        Transfer dead biomass to litter and update stand density due
!!              to thinning operations
!!
!! DESCRIPTION  : Moves dead biomass to the litter pool and the harvest pool
!!                where appropriate.  Since not all the trees are killed,
!!                fewer counters have to be reset here than in the clearcut
!!                case.  In addition, thinning will change the fraction of
!!                trees killed by mortality, so we account for that as well.
!!
!!                NOTE: If harvest_fraction is 1, all the aboveground biomass is
!!                      harvested.  If it's 1-branch_ratio, only stems are 
!!                      harvested.  Belowground wood, roots, fruits, and
!!                      leaves all become litter.  A portion of aboveground
!!                      wood is put in the harvest pool.  Reserves and labile
!!                      carbon are assumed to be distributed evenly across
!!                      all non-heartwood components.
!!
!! RECENT CHANGE(S): 
!!
!! MAIN OUTPUT VARIABLE(S): ::circ_class_biomass; biomass, ::ind density of individuals, 
!! ::circ_class_kill
!!
!! REFERENCE(S)   :
!!
!! FLOWCHART    : None
!!\n
!_ ================================================================================================================================
 
  SUBROUTINE thinning_harvest_aboveground_dead_roots (npts, ipts, ivm, iele, &
       ifm, icut, biomass, ind, bm_to_litter, &
       circ_class_biomass, circ_class_kill, circ_class_n, harvest_pool, &
       harvest_type, harvest_cut, harvest_area, &
       harvest_pool_bound, resolution, veget_max, last_cut, harvest_fraction)


    !! 0. Variable and parameter declaration

    !! 0.1 Input variables 
    INTEGER(i_std), INTENT(in)                                   :: npts               !! Domain size (-)
    INTEGER(i_std), INTENT(in)                                   :: ipts               !! current grid square
    INTEGER(i_std), INTENT(in)                                   :: ivm                !! current PFT
    INTEGER(i_std), INTENT(in)                                   :: iele               !! current element type
    INTEGER(i_std), INTENT(in)                                   :: ifm                !! current fm strategy
    INTEGER(i_std), INTENT(in)                                   :: icut               !! current cut time
    REAL(r_std), DIMENSION(0:ndia_harvest+1), INTENT(in)         :: harvest_pool_bound !! The boundaries of the diameter classes
                                                                                       !! in the wood harvest pools
                                                                                       !! @tex $(m)$ @endtex
    REAL(r_std),DIMENSION(npts,2), INTENT(in)                    :: resolution         !! The length of each grid-box in X and Y
                                                                                       !! @tex $(m)$ @endtex
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)                   :: veget_max          !! Maximum fraction of vegetation type 
    REAL(r_std),INTENT(in)                                       :: harvest_fraction   !! The fraction of the aboveground
                                                                                       !! biomass that is harvested.  This is
                                                                                       !! generally 1-branch_ratio if you only
                                                                                       !! want to harvest the stems, and 1 if
                                                                                       !! you want to harvest everything.


    !! 0.2 Output variables


    !! 0.3 Modified variables   

    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                             INTENT(inout)       :: biomass            !! Biomass @tex $(gC m^{-2}) $@endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)              :: ind                !! Stand level number of individuals 
                                                                                       !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nparts,nelements), &                         
                                              INTENT(inout)      :: circ_class_biomass !! Biomass of the components of the model  
                                                                                       !! tree within a circumference
                                                                                       !! class @tex $(gC ind^{-1})$ @endtex  
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(inout)        :: circ_class_n       !! Number of individuals in each circ class
                                                                                       !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nfm_types,ncut_times), &
                                                  INTENT(inout)  :: circ_class_kill    !! Number of trees within a circ that needs
                                                                                       !! to be killed @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                                  INTENT(inout)  :: bm_to_litter       !! Biomass transfer to litter 
                                                                                       !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)               :: harvest_pool       !! The wood and biomass that have been
                                                                                       !! havested by humans @tex $(gC)$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)                   :: harvest_type       !! Type of management that resulted
                                                                                       !! in the harvest (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)                   :: harvest_cut        !! Type of cutting that was used for the harvest
                                                                                       !! (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)                   :: harvest_area       !! Harvested area (m^{2})
    INTEGER(i_std), DIMENSION(npts,nvm),INTENT(inout)            :: last_cut           !! Years since last thinning or clearcut (years)

    !! 0.4 Local variables

    INTEGER(i_std)                                               :: icir               !! Indices
    REAL(r_std)                                                  :: total_bm
    REAL(r_std)                                                  :: thinned_bm
    REAL(r_std)                                                  :: dead_bm
    REAL(r_std)                                                  :: bm_difference

    !_ ================================================================================================================================

    IF (bavard.GE.2) WRITE(numout,*) &
         'Entering thinning_harvest_aboveground_dead_roots sapiens_kill.f90'

    IF( SUM(circ_class_kill(ipts,ivm,:,ifm,icut)) .GT. min_stomate ) THEN

       CALL harvest_aboveground_dead_roots(npts, ipts, ivm, iele, &
            ifm, icut, biomass, ind, bm_to_litter, &
            circ_class_biomass, circ_class_kill, circ_class_n, harvest_pool, &
            harvest_type, harvest_cut, harvest_area, &
            harvest_pool_bound, resolution, veget_max, harvest_fraction)

       ! thinning trees generally avoids natural mortality and self-thinning, so
       ! let's remove all thinned trees from the pools.  This is tricky because
       ! mortality depends on a fraction of the total biomass, which may be spread
       ! out over different circ classes than what were killed by thinning.
       thinned_bm = zero
       dead_bm = zero
       DO icir=1,ncirc
          total_bm=SUM(circ_class_biomass(ipts,ivm,icir,:,iele))
          thinned_bm=thinned_bm+circ_class_kill(ipts,ivm,icir,ifm,icut)*total_bm
          dead_bm=dead_bm+circ_class_kill(ipts,ivm,icir,ifm_none,icut_thin)*total_bm
       ENDDO
       bm_difference = dead_bm - thinned_bm

       ! if the thinned biomass is more than what would have died from natural causes,
       ! we can just assume that thinning killed all these trees.
       IF(bm_difference .LE. min_stomate)THEN

          circ_class_kill(ipts,ivm,:,ifm_none,icut_thin)=zero

       ELSE

          ! if not, we have to reduce the amount of biomass that is killed naturally
          ! by what was removed by thinning, redistributing the remainder properly
          ! across all circ classes as we did in stomate_mark_kill.
          circ_class_kill(ipts,ivm,:,ifm_none,icut_thin) = zero
          CALL distribute_mortality_biomass( bm_difference, &
               death_distribution_factor(ivm), &
               circ_class_n(ipts,ivm,:), &
               circ_class_biomass(ipts,ivm,:,:,icarbon), &
               circ_class_kill(ipts,ivm,:,ifm_none,icut_thin) )
       ENDIF ! bm_difference .LE. min_stomate


       ! make sure nothing has gone negative
       WHERE ( circ_class_kill(ipts,ivm,:,ifm_none,icut_thin) .LT. zero )
          circ_class_kill(ipts,ivm,:,ifm_none,icut_thin)=zero
       END WHERE

       ! now zero out this circ_class_kill, since we should never try
       ! to kill these same trees again.
       circ_class_kill(ipts,ivm,:,ifm,icut)=zero

       ! Reset the values for the time since the last operation
       last_cut(ipts,ivm)=0
       
    ENDIF ! checking to see if we have biomass to thin


    IF (bavard.GE.2) WRITE(numout,*) &
         'Leaving thinning_harvest_aboveground_dead_roots sapiens_kill.f90'

  END SUBROUTINE thinning_harvest_aboveground_dead_roots

!! ================================================================================================================================
!! SUBROUTINE   : harvest_aboveground_dead_roots
!!
!>\BRIEF        Transfer dead biomass to litter and wood to harvest pool.
!!
!! DESCRIPTION  : Moves dead biomass to the litter pool and the harvest pool
!!                where appropriate.  
!!
!!                NOTE: If harvest_fraction is 1, all the aboveground biomass is
!!                      harvested.  If it's 1-branch_ratio, only stems are 
!!                      harvested.  Belowground wood, roots, fruits, and
!!                      leaves all become litter.  A portion of aboveground
!!                      wood is put in the harvest pool.  Reserves and labile
!!                      carbon are assumed to be distributed evenly across
!!                      all non-heartwood components.
!!
!! RECENT CHANGE(S): 
!!
!! MAIN OUTPUT VARIABLE(S): ::bm_to_litter, ::harvest_pool
!!
!! REFERENCE(S)   :
!!
!! FLOWCHART    : None
!!\n
!_ ================================================================================================================================
 
  SUBROUTINE harvest_aboveground_dead_roots (npts, ipts, ivm, iele, &
       ifm, icut, biomass, ind, bm_to_litter, &
       circ_class_biomass, circ_class_kill, circ_class_n, harvest_pool, &
       harvest_type, harvest_cut, harvest_area, &
       harvest_pool_bound, resolution, veget_max, harvest_fraction)


    !! 0. Variable and parameter declaration

    !! 0.1 Input variables 

    INTEGER(i_std), INTENT(in)                                   :: npts               !! Domain size (-)
    INTEGER(i_std), INTENT(in)                                   :: ipts               !! current grid square
    INTEGER(i_std), INTENT(in)                                   :: ivm                !! current PFT
    INTEGER(i_std), INTENT(in)                                   :: iele               !! current element type
    INTEGER(i_std), INTENT(in)                                   :: ifm                !! current fm strategy
    INTEGER(i_std), INTENT(in)                                   :: icut               !! current cut time
    REAL(r_std), DIMENSION(0:ndia_harvest+1), INTENT(in)         :: harvest_pool_bound !! The boundaries of the diameter classes
                                                                                       !! in the wood harvest pools
                                                                                       !! @tex $(m)$ @endtex
    REAL(r_std),DIMENSION(npts,2), INTENT(in)                    :: resolution         !! The length of each grid-box in X and Y
                                                                                       !! @tex $(m)$ @endtex
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)                   :: veget_max          !! Maximum fraction of vegetation type 
    REAL(r_std),INTENT(in)                                       :: harvest_fraction   !! The fraction of the aboveground
                                                                                       !! biomass that is harvested.  This is
                                                                                       !! generally 1-branch_ratio if you only
                                                                                       !! want to harvest the stems, and 1 if
                                                                                       !! you want to harvest everything.

    !! 0.2 Output variables


    !! 0.3 Modified variables   

    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                             INTENT(inout)       :: biomass            !! Biomass @tex $(gC m^{-2}) $@endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)              :: ind                !! Stand level number of individuals 
                                                                                       !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nparts,nelements), &                         
                                              INTENT(inout)      :: circ_class_biomass !! Biomass of the components of the model  
                                                                                       !! tree within a circumference
                                                                                       !! class @tex $(gC ind^{-1})$ @endtex  
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(inout)        :: circ_class_n       !! Number of individuals in each circ class
                                                                                       !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nfm_types,ncut_times), &
                                                  INTENT(inout)  :: circ_class_kill    !! Number of trees within a circ that needs
                                                                                       !! to be killed @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                                  INTENT(inout)  :: bm_to_litter       !! Biomass transfer to litter 
                                                                                       !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)               :: harvest_pool       !! The wood and biomass that have been
                                                                                       !! havested by humans @tex $(gC)$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)                   :: harvest_type       !! Type of management that resulted
                                                                                       !! in the harvest (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)                   :: harvest_cut        !! Type of cutting that was used for the harvest
                                                                                       !! (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)                   :: harvest_area       !! Harvested area (m^{2})


    !! 0.4 Local variables

    INTEGER(i_std)                                               :: idia, icir         !! Indices
    REAL(r_std)                                                  :: total_nonheart_biomass_harvest
    REAL(r_std)                                                  :: total_nonheart_biomass_kill
    REAL(r_std)                                                  :: harvest_ratio
    REAL(r_std), DIMENSION(ncirc)                                :: diameters_temp

    !_ ================================================================================================================================

    IF (bavard.GE.2) WRITE(numout,*) &
         'Entering harvest_aboveground_dead_roots sapiens_kill.f90'

    ! we will need the trunk diameters for all the circ classes in order to figure
    ! out which harvest pool to put them in.
    diameters_temp(:)=wood_to_dia(circ_class_biomass(ipts,ivm,:,:,icarbon),ivm) 

    DO icir=1,ncirc

       ! It's possible that we only kill individuals in certain circ classes.
       ! In this case, we need to check for each circ class to make sure
       ! we don't divide by zero.
       IF(circ_class_kill(ipts,ivm,icir,ifm,icut) .LT. min_stomate) CYCLE

       ! The decision of what to do with the carboyhydrate reserves and the
       ! labile carbon pool are tricky.  In turnover, they are not touched
       ! when leaves/roots/fruits die, which implies that they are stored
       ! primarily in the above and belowground sapwood (heartwood is dead). 
       ! However, here a tree does not know thinning is coming, so it cannot
       ! move this carbon around.  We will therefore determine where is goes
       ! based on the total amount of non-heartwood pools and where they go.
       total_nonheart_biomass_kill=circ_class_kill(ipts,ivm,icir,ifm,icut)*&
            (circ_class_biomass(ipts,ivm,icir,isapbelow,iele)+&
            circ_class_biomass(ipts,ivm,icir,iroot,iele)+&
            circ_class_biomass(ipts,ivm,icir,ileaf,iele)+&
            circ_class_biomass(ipts,ivm,icir,ifruit,iele)+&
            circ_class_biomass(ipts,ivm,icir,isapabove,iele))

       total_nonheart_biomass_harvest=harvest_fraction*&
            circ_class_kill(ipts,ivm,icir,ifm,icut)*&
            circ_class_biomass(ipts,ivm,icir,isapabove,iele)

       ! This is now the fraction of the nonheartwood biomass that will go
       ! into the harvest pool.  We will use this to partition the carbohydrate
       ! reserve and labile pools into litter and harvest.
       harvest_ratio=total_nonheart_biomass_harvest/total_nonheart_biomass_kill

       ! all of the following biomass becomes litter
       bm_to_litter(ipts,ivm,isapbelow,iele) = &
            bm_to_litter(ipts,ivm,isapbelow,iele) + &
            circ_class_biomass(ipts,ivm,icir,isapbelow,iele)*&
            circ_class_kill(ipts,ivm,icir,ifm,icut)
       bm_to_litter(ipts,ivm,iheartbelow,iele) = &
            bm_to_litter(ipts,ivm,iheartbelow,iele) + &
            circ_class_biomass(ipts,ivm,icir,iheartbelow,iele)*&
            circ_class_kill(ipts,ivm,icir,ifm,icut)
       bm_to_litter(ipts,ivm,iroot,iele) = bm_to_litter(ipts,ivm,iroot,iele) + &
            circ_class_biomass(ipts,ivm,icir,iroot,iele)*&
            circ_class_kill(ipts,ivm,icir,ifm,icut)
       bm_to_litter(ipts,ivm,ileaf,iele) = bm_to_litter(ipts,ivm,ileaf,iele) + &
            circ_class_biomass(ipts,ivm,icir,ileaf,iele)*&
            circ_class_kill(ipts,ivm,icir,ifm,icut)
       bm_to_litter(ipts,ivm,ifruit,iele) = bm_to_litter(ipts,ivm,ifruit,iele) + &
            circ_class_biomass(ipts,ivm,icir,ifruit,iele)*&
            circ_class_kill(ipts,ivm,icir,ifm,icut)

       ! Include the branches as litter, if necessary.
       bm_to_litter(ipts,ivm,isapabove,iele) = &
            bm_to_litter(ipts,ivm,isapabove,iele) + &
            (un - harvest_fraction)*circ_class_biomass(ipts,ivm,icir,isapabove,iele)*&
            circ_class_kill(ipts,ivm,icir,ifm,icut)
       bm_to_litter(ipts,ivm,iheartabove,iele) = &
            bm_to_litter(ipts,ivm,iheartabove,iele) + &
            (un - harvest_fraction)*circ_class_biomass(ipts,ivm,icir,iheartabove,iele)*&
            circ_class_kill(ipts,ivm,icir,ifm,icut)

       ! Don't forget to include the proper proportion of the labile and
       ! carbres pools.
       bm_to_litter(ipts,ivm,icarbres,iele) = &
            bm_to_litter(ipts,ivm,icarbres,iele) + &
            (un - harvest_ratio )*circ_class_biomass(ipts,ivm,icir,icarbres,iele)*&
            circ_class_kill(ipts,ivm,icir,ifm,icut)
       bm_to_litter(ipts,ivm,ilabile,iele) = &
            bm_to_litter(ipts,ivm,ilabile,iele) + &
            (un - harvest_ratio )*circ_class_biomass(ipts,ivm,icir,ilabile,iele)*&
            circ_class_kill(ipts,ivm,icir,ifm,icut)

       ! now we have the stems.  These are what we want to save, and we are
       ! going to classify them by PFT type, location, and diameter so that
       ! we can compare to FAO statistics and do some more interesting
       ! post-treatment.

       ! UNITS: The units of the biomass pools are per meter squared.
       ! It seems to make more sense to store the harvest pool in absolute
       ! units, which means we have to multiple the result by the fraction of
       ! this grid cell covered by this PFT (veget_max) and the size of the 
       ! grid cell.  It seems that the fraction of nobio land does not need 
       ! to be used here, since frac_nobio and veget_max are summed together 
       ! in slowproc and they must equal one.  This gives
       ! us the grams of carbon harvest, which is a big number but that's
       ! why people invented scientific notation.

       dia_loop:DO idia=1,ndia_harvest

          IF(diameters_temp(icir) .LE. harvest_pool_bound(idia))THEN

             ! this is the correct diameter class 
             harvest_pool(ipts,ivm,idia,icarbon) = &
                  harvest_pool(ipts,ivm,idia,icarbon) + &
                  harvest_fraction*circ_class_kill(ipts,ivm,icir,ifm,icut)*&
                  (circ_class_biomass(ipts,ivm,icir,isapabove,iele)+&
                  circ_class_biomass(ipts,ivm,icir,iheartabove,iele)) * &
                  veget_max(ipts,ivm) * &
                  resolution(ipts,1) * resolution(ipts,2)

             ! also add the proper fractions of the labile and carbres pools
             harvest_pool(ipts,ivm,idia,icarbon) = &
                  harvest_pool(ipts,ivm,idia,icarbon)+ &
                  harvest_ratio * circ_class_kill(ipts,ivm,icir,ifm,icut)*&
                  ( circ_class_biomass(ipts,ivm,icir,ilabile,iele) + &
                  circ_class_biomass(ipts,ivm,icir,icarbres,iele) )* &
                  veget_max(ipts,ivm) * &
                  resolution(ipts,1) * resolution(ipts,2)
             
             ! Associated harvest variables
             harvest_type(ipts,ivm) = ifm
             harvest_cut(ipts,ivm) = icut
             harvest_area(ipts,ivm) = veget_max(ipts,ivm) * &
                  resolution(ipts,1) * resolution(ipts,2)
                
             EXIT dia_loop

          ELSEIF ( (diameters_temp(icir) .GT. harvest_pool_bound(idia)) .AND. &
               (diameters_temp(icir) .LT. harvest_pool_bound(idia+1)) .AND. &
               (idia == ndia_harvest) ) THEN
             
             ! Store in largest diameter class 
             harvest_pool(ipts,ivm,idia,icarbon) = &
                  harvest_pool(ipts,ivm,idia,icarbon) + &
                  harvest_fraction*circ_class_kill(ipts,ivm,icir,ifm,icut)*&
                  (circ_class_biomass(ipts,ivm,icir,isapabove,iele)+&
                  circ_class_biomass(ipts,ivm,icir,iheartabove,iele)) * &
                  veget_max(ipts,ivm) * &
                  resolution(ipts,1) * resolution(ipts,2)

             ! also add the proper fractions of the labile and carbres pools
             harvest_pool(ipts,ivm,idia,icarbon) = &
                  harvest_pool(ipts,ivm,idia,icarbon)+ &
                  harvest_ratio * circ_class_kill(ipts,ivm,icir,ifm,icut)*&
                  ( circ_class_biomass(ipts,ivm,icir,ilabile,iele) + &
                  circ_class_biomass(ipts,ivm,icir,icarbres,iele) )* &
                  veget_max(ipts,ivm) * &
                  resolution(ipts,1) * resolution(ipts,2)
             
             ! Associated harvest variables
             harvest_type(ipts,ivm) = ifm
             harvest_cut(ipts,ivm) = icut
             harvest_area(ipts,ivm) = veget_max(ipts,ivm) * &
                  resolution(ipts,1) * resolution(ipts,2)
                
             EXIT dia_loop

          ENDIF ! diameters_temp(icir) .LE. harvest_pool_bound(idia)

          ! If we are here, it means the diameter of this trunk is larger
          ! than the upper boundary of our harvest bins.  Considering the
          ! upper boundary is set to be 99999 meters in constants.f90,
          ! something has gone wrong!
          IF (diameters_temp(icir) .GT. harvest_pool_bound(ndia_harvest+1)) THEN

             WRITE(numout,*) 'Our tree is too big to fit into the harvest pools.'
             WRITE(numout,*) 'ipts, ivm, icir: ',ipts, ivm, icir
             WRITE(numout,*) 'diameters_temp(icir), harvest_pool_bound(idia)',&
                  diameters_temp(icir),harvest_pool_bound(idia)
             STOP

          ENDIF

       ENDDO dia_loop


       ! remove the number of individuals that died
       circ_class_n(ipts,ivm,icir) = circ_class_n(ipts,ivm,icir) - &
            circ_class_kill(ipts,ivm,icir,ifm,icut)

       ! remove the dead biomass
       biomass(ipts,ivm,:,iele) = biomass(ipts,ivm,:,iele) - &
            circ_class_biomass(ipts,ivm,icir,:,iele)*&
            circ_class_kill(ipts,ivm,icir,ifm,icut)

       ! adjust the number of individuals
       ind(ipts,ivm)=ind(ipts,ivm)-circ_class_kill(ipts,ivm,icir,ifm,icut)

    ENDDO ! loop over circ classes

    ! We should not have negative numbers.  I don't want to zero out all
    ! of them, because it could be sign of a real problem, but if they
    ! are a very small negative number it's just a numerical issue.
    WHERE( ABS(biomass(ipts,ivm,:,iele)) .LE. min_stomate)
       biomass(ipts,ivm,:,iele) = zero
    END WHERE
    WHERE( ABS(circ_class_biomass(ipts,ivm,:,:,iele)) .LE. min_stomate)
      circ_class_biomass(ipts,ivm,:,:,iele) = zero
    END WHERE
    WHERE( ABS(circ_class_n(ipts,ivm,:)) .LE. min_stomate)
       circ_class_n(ipts,ivm,:) = zero
    END WHERE

    IF (bavard.GE.2) WRITE(numout,*) &
         'Leaving harvest_aboveground_dead_roots sapiens_kill.f90'

  END SUBROUTINE harvest_aboveground_dead_roots

!! ================================================================================================================================
!! SUBROUTINE   : harvest_aboveground_wood_living_roots
!!
!>\BRIEF        Transfer dead biomass to litter and wood to harvest pool.
!!
!! DESCRIPTION  : Moves dead biomass to the litter pool and the harvest pool
!!                where appropriate.  
!!
!!                NOTE: All branches and stems are harvested.  Basically, this means
!!                      all aboveground woody biomass.  The fine roots
!!                      are killed and left to litter.  The sapwood and heartwood
!!                      belowground is left alive.  The leaves are killed, and the
!!                      labile and carbres pools are left untouched in the living
!!                      biomass.
!!
!! RECENT CHANGE(S): 
!!
!! MAIN OUTPUT VARIABLE(S): ::bm_to_litter, ::harvest_pool
!!
!! REFERENCE(S)   :
!! - Bellassen, V., Le Maire, G., Dhote, J.F., Viovy, N., Ciais, P., 2010. 
!! Modeling forest management within a global vegetation model – Part 1: 
!! model structure and general behaviour. Ecological Modelling 221, 2458–2474.
!!
!! FLOWCHART    : None
!!\n
!_ ================================================================================================================================
 
  SUBROUTINE harvest_aboveground_wood_living_roots (npts, ipts, ivm, iele, &
       ifm, icut, biomass, bm_to_litter, &
       circ_class_biomass, circ_class_kill, harvest_pool, &
       harvest_type, harvest_cut, harvest_area, &
       harvest_pool_bound, resolution, veget_max)


    !! 0. Variable and parameter declaration

    !! 0.1 Input variables 
    INTEGER(i_std), INTENT(in)                                   :: npts               !! Domain size (-)
    INTEGER(i_std), INTENT(in)                                   :: ipts               !! current grid square
    INTEGER(i_std), INTENT(in)                                   :: ivm                !! current PFT
    INTEGER(i_std), INTENT(in)                                   :: iele               !! current element type
    INTEGER(i_std), INTENT(in)                                   :: ifm                !! current fm strategy
    INTEGER(i_std), INTENT(in)                                   :: icut               !! current cut time
    REAL(r_std), DIMENSION(0:ndia_harvest+1), INTENT(in)         :: harvest_pool_bound !! The boundaries of the diameter classes
                                                                                       !! in the wood harvest pools
                                                                                       !! @tex $(m)$ @endtex
    REAL(r_std),DIMENSION(npts,2), INTENT(in)                    :: resolution         !! The length of each grid-box in X and Y
                                                                                       !! @tex $(m)$ @endtex
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)                   :: veget_max          !! Maximum fraction of vegetation type 


    !! 0.2 Output variables


    !! 0.3 Modified variables   

    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                             INTENT(inout)       :: biomass            !! Biomass @tex $(gC m^{-2}) $@endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nparts,nelements), &                         
                                              INTENT(inout)      :: circ_class_biomass !! Biomass of the components of the model  
                                                                                       !! tree within a circumference
                                                                                       !! class @tex $(gC ind^{-1})$ @endtex  
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nfm_types,ncut_times), &
                                                  INTENT(inout)  :: circ_class_kill    !! Number of trees within a circ that needs
                                                                                       !! to be killed @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                                  INTENT(inout)  :: bm_to_litter       !! Biomass transfer to litter 
                                                                                       !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)               :: harvest_pool       !! The wood and biomass that have been
                                                                                       !! havested by humans @tex $(gC)$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)                   :: harvest_type       !! Type of management that resulted
                                                                                       !! in the harvest (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)                   :: harvest_cut        !! Type of cutting that was used for the harvest
                                                                                       !! (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)                   :: harvest_area       !! Harvested area (m^{2})

    !! 0.4 Local variables

    INTEGER(i_std)                                               :: idia, icir         !! Indices
    REAL(r_std)                                                  :: total_nonheart_biomass_harvest
    REAL(r_std)                                                  :: total_nonheart_biomass_living
    REAL(r_std)                                                  :: total_nonheart_biomass_kill
    REAL(r_std)                                                  :: harvest_ratio
    REAL(r_std)                                                  :: living_ratio
    REAL(r_std)                                                  :: litter_ratio
    REAL(r_std), DIMENSION(ncirc)                                :: diameters_temp

    !_ ================================================================================================================================

    IF (bavard.GE.2) WRITE(numout,*) &
         'Entering harvest_aboveground_wood_living_roots sapiens_kill.f90'

    ! we will need the trunk diameters for all the circ classes in order to figure
    ! out which harvest pool to put them in.
    diameters_temp(:)=wood_to_dia(circ_class_biomass(ipts,ivm,:,:,icarbon),ivm) 

    DO icir=1,ncirc

       ! It's possible that we only kill individuals in certain circ classes.
       ! In this case, we need to check for each circ class to make sure
       ! we don't divide by zero.
       IF(circ_class_kill(ipts,ivm,icir,ifm,icut) .LT. min_stomate) CYCLE

       ! The decision of what to do with the carboyhydrate reserves and the
       ! labile carbon pool are tricky.  In turnover, they are not touched
       ! when leaves/roots/fruits die, which implies that they are stored
       ! primarily in the above and belowground sapwood (heartwood is dead). 
       ! However, here a tree does not know thinning is coming, so it cannot
       ! move this carbon around.  In theory, we should remove part of the
       ! labile and reserve pools to the different product pools, but
       ! these pools are very sensitive for the allocation.  Therefore
       ! we don't touch them, which is equivalent to saying the reserve
       ! and labile carbon are all found in the roots.

       total_nonheart_biomass_kill=circ_class_kill(ipts,ivm,icir,ifm,icut)*&
            (circ_class_biomass(ipts,ivm,icir,isapbelow,iele)+&
            circ_class_biomass(ipts,ivm,icir,iroot,iele)+&
            circ_class_biomass(ipts,ivm,icir,ileaf,iele)+&
            circ_class_biomass(ipts,ivm,icir,ifruit,iele)+&
            circ_class_biomass(ipts,ivm,icir,isapabove,iele))

       total_nonheart_biomass_harvest=&
            circ_class_kill(ipts,ivm,icir,ifm,icut)*&
            (circ_class_biomass(ipts,ivm,icir,isapabove,iele))

       total_nonheart_biomass_living=&
            circ_class_kill(ipts,ivm,icir,ifm,icut)*&
            (circ_class_biomass(ipts,ivm,icir,isapbelow,iele))

       ! This is now the fraction of the nonheartwood biomass that will go
       ! into the harvest pool.  We will use this to partition the carbohydrate
       ! reserve and labile pools into litter and harvest.
       harvest_ratio=total_nonheart_biomass_harvest/total_nonheart_biomass_kill
       living_ratio=total_nonheart_biomass_living/total_nonheart_biomass_kill
       litter_ratio=un-harvest_ratio-living_ratio
       IF(litter_ratio .LT. -min_stomate)THEN
          WRITE(numout,*) 'We have a problem with coppicing!'
          WRITE(numout,*) 'ipts,ivm,icir,ifm,icut ',ipts,ivm,icir,ifm,icut
          WRITE(numout,*) 'harvest_ratio, living_ratio, litter_ratio ',&
               harvest_ratio, living_ratio, litter_ratio
          STOP
       ELSEIF(litter_ratio .LT. -min_stomate)THEN
          litter_ratio=zero
       ENDIF

       ! all of the following biomass becomes litter
       bm_to_litter(ipts,ivm,iroot,iele) = bm_to_litter(ipts,ivm,iroot,iele) + &
            circ_class_biomass(ipts,ivm,icir,iroot,iele)*&
            circ_class_kill(ipts,ivm,icir,ifm,icut)
       bm_to_litter(ipts,ivm,ileaf,iele) = bm_to_litter(ipts,ivm,ileaf,iele) + &
            circ_class_biomass(ipts,ivm,icir,ileaf,iele)*&
            circ_class_kill(ipts,ivm,icir,ifm,icut)
       bm_to_litter(ipts,ivm,ifruit,iele) = bm_to_litter(ipts,ivm,ifruit,iele) + &
            circ_class_biomass(ipts,ivm,icir,ifruit,iele)*&
            circ_class_kill(ipts,ivm,icir,ifm,icut)

       ! now we have the stems.  These are what we want to save, and we are
       ! going to classify them by PFT type, location, and diameter so that
       ! we can compare to FAO statistics and do some more interesting
       ! post-treatment.

       ! UNITS: The units of the biomass pools are per meter squared.
       ! It seems to make more sense to store the harvest pool in absolute
       ! units, which means we have to multiple the result by the fraction of
       ! this grid cell covered by this PFT (veget_max) and the size of the 
       ! grid cell.  It seems that the fraction of nobio land does not need 
       ! to be used here, since frac_nobio and veget_max are summed together 
       ! in slowproc and they must equal one.  This gives
       ! us the grams of carbon harvest, which is a big number but that's
       ! why people invented scientific notation.

       dia_loop:DO idia=1,ndia_harvest

          IF(diameters_temp(icir) .LE. harvest_pool_bound(idia))THEN

             ! this is the correct diameter class 
             harvest_pool(ipts,ivm,idia,icarbon)=&
                  harvest_pool(ipts,ivm,idia,icarbon)+ &
                  circ_class_kill(ipts,ivm,icir,ifm,icut)*&
                  (circ_class_biomass(ipts,ivm,icir,isapabove,iele)+&
                  circ_class_biomass(ipts,ivm,icir,iheartabove,iele)) * &
                  veget_max(ipts,ivm) * &
                  resolution(ipts,1) * resolution(ipts,2)
             
             ! Associated harvest variables
             harvest_type(ipts,ivm) = ifm
             harvest_cut(ipts,ivm) = icut
             harvest_area(ipts,ivm) = veget_max(ipts,ivm) * &
                  resolution(ipts,1) * resolution(ipts,2)

             EXIT dia_loop

          ENDIF ! diameters_temp(icir) .LE. harvest_pool_bound(idia)

          ! If we are here, it means the diameter of this trunk is larger
          ! than the upper boundary of our harvest bins.  Considering the
          ! upper boundary is set to be 99999 meters in constants.f90,
          ! something has gone wrong!
          IF(idia == ndia_harvest)THEN

             WRITE(numout,*) 'Our tree is too big to fit into the harvest pools.'
             WRITE(numout,*) 'ipts, ivm, icir: ',ipts, ivm, icir
             WRITE(numout,*) 'diameters_temp(icir), harvest_pool_bound(idia)',&
                  diameters_temp(icir),harvest_pool_bound(idia)
             STOP

          ENDIF

       ENDDO dia_loop


       ! No individuals die here since the roots are still alive.

       ! Remove the dead biomass.  This includes everything except
       ! the belowground biomass.  
       biomass(ipts,ivm,ileaf,iele) = biomass(ipts,ivm,ileaf,iele) - &
            circ_class_biomass(ipts,ivm,icir,ileaf,iele)*&
            circ_class_kill(ipts,ivm,icir,ifm,icut)
       biomass(ipts,ivm,isapabove,iele) = biomass(ipts,ivm,isapabove,iele) - &
            circ_class_biomass(ipts,ivm,icir,isapabove,iele)*&
            circ_class_kill(ipts,ivm,icir,ifm,icut)
       biomass(ipts,ivm,iheartabove,iele) = biomass(ipts,ivm,iheartabove,iele) - &
            circ_class_biomass(ipts,ivm,icir,iheartabove,iele)*&
            circ_class_kill(ipts,ivm,icir,ifm,icut)
       biomass(ipts,ivm,iroot,iele) = biomass(ipts,ivm,iroot,iele) - &
            circ_class_biomass(ipts,ivm,icir,iroot,iele)*&
            circ_class_kill(ipts,ivm,icir,ifm,icut)
       biomass(ipts,ivm,ifruit,iele) = biomass(ipts,ivm,ifruit,iele) - &
            circ_class_biomass(ipts,ivm,icir,ifruit,iele)*&
            circ_class_kill(ipts,ivm,icir,ifm,icut)

       ! Remove the dead biomass.  This includes everything except
       ! the belowground biomass.  
       circ_class_biomass(ipts,ivm,icir,ileaf,iele)=zero
       circ_class_biomass(ipts,ivm,icir,isapabove,iele)=zero
       circ_class_biomass(ipts,ivm,icir,iheartabove,iele)=zero
       circ_class_biomass(ipts,ivm,icir,iroot,iele)=zero
       circ_class_biomass(ipts,ivm,icir,ifruit,iele)=zero

    ENDDO ! loop over circ classes


    IF (bavard.GE.2) WRITE(numout,*) &
         'Leaving harvest_aboveground_wood_living_roots sapiens_kill.f90'

  END SUBROUTINE harvest_aboveground_wood_living_roots

!! ================================================================================================================================
!! SUBROUTINE   : second_coppice_cut
!!
!>\BRIEF        Simulates cutting a coppice between the first cut and the
!!              final cut.
!!
!! DESCRIPTION  : Moves dead biomass to the litter pool and the harvest pool
!!                where appropriate.  A few counters are reset.  The leftover
!!                biomass (the woody root system) is patitioned among enough
!!                individuals to give the same density as is found in coppice_dens,
!!                which is the density of stumps left after the first cut.
!!
!!                NOTE: All branches and stems are harvested.  Basically, this means
!!                      all aboveground woody biomass.  The fine roots
!!                      are killed and left to litter.  The sapwood and heartwood
!!                      belowground is left alive.  The leaves are killed, and the
!!                      labile and carbres pools are left untouched in the living
!!                      biomass.
!!
!! RECENT CHANGE(S): 
!!
!! MAIN OUTPUT VARIABLE(S): ::bm_to_litter, ::harvest_pool
!!
!! REFERENCE(S)   :
!! - Bellassen, V., Le Maire, G., Dhote, J.F., Viovy, N., Ciais, P., 2010. 
!! Modeling forest management within a global vegetation model – Part 1: 
!! model structure and general behaviour. Ecological Modelling 221, 2458–2474.
!!
!! FLOWCHART    : None
!!\n
!_ ================================================================================================================================
 
  SUBROUTINE second_coppice_cut (npts, ipts, ivm, iele, ifm, &
       icut, harvest_pool_bound, resolution, veget_max, biomass, &
       ind, circ_class_biomass, circ_class_n, circ_class_kill, &
       bm_to_litter, harvest_pool, harvest_type, harvest_cut, &
       harvest_area, circ_class_dist, last_cut, coppice_dens)


    !! 0. Variable and parameter declaration

    !! 0.1 Input variables 
    INTEGER(i_std), INTENT(in)                                   :: npts               !! Domain size (-)
    INTEGER(i_std), INTENT(in)                                   :: ipts               !! current grid square
    INTEGER(i_std), INTENT(in)                                   :: ivm                !! current PFT
    INTEGER(i_std), INTENT(in)                                   :: iele               !! current element type
    INTEGER(i_std), INTENT(in)                                   :: ifm                !! current fm strategy
    INTEGER(i_std), INTENT(in)                                   :: icut               !! current cut time
    REAL(r_std), DIMENSION(0:ndia_harvest+1), INTENT(in)         :: harvest_pool_bound !! The boundaries of the diameter classes
                                                                                       !! in the wood harvest pools
                                                                                       !! @tex $(m)$ @endtex
    REAL(r_std),DIMENSION(npts,2), INTENT(in)                    :: resolution         !! The length of each grid-box in X and Y
                                                                                       !! @tex $(m)$ @endtex
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)                   :: veget_max          !! Maximum fraction of vegetation type 
    REAL(r_std), DIMENSION(ncirc), INTENT(in)                    :: circ_class_dist    !! The probability distribution of trees 
                                                                                       !! in a circ class in case of a 
                                                                                       !! redistribution (unitless).
    REAL(r_std), DIMENSION(npts,nvm),INTENT(in)                  :: coppice_dens       !! The density of a coppice at the first
                                                                                       !! cutting.
                                                                                       !! @tex $( 1 / m**2 )$ @endtex 

    !! 0.2 Output variables


    !! 0.3 Modified variables   
    
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                             INTENT(inout)       :: biomass            !! Biomass @tex $(gC m^{-2}) $@endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)              :: ind                !! Stand level number of individuals 
                                                                                       !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nparts,nelements), &                         
                                              INTENT(inout)      :: circ_class_biomass !! Biomass of the components of the model  
                                                                                       !! tree within a circumference
                                                                                       !! class @tex $(gC ind^{-1})$ @endtex  
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(inout)        :: circ_class_n       !! Number of individuals in each circ class
                                                                                       !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nfm_types,ncut_times), &
                                                  INTENT(inout)  :: circ_class_kill    !! Number of trees within a circ that needs
                                                                                       !! to be killed @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                                  INTENT(inout)  :: bm_to_litter       !! Biomass transfer to litter 
                                                                                       !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)               :: harvest_pool       !! The wood and biomass that have been
                                                                                       !! havested by humans @tex $(gC)$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)                   :: harvest_type       !! Type of management that resulted
                                                                                       !! in the harvest (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)                   :: harvest_cut        !! Type of cutting that was used for the harvest
                                                                                       !! (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)                   :: harvest_area       !! Harvested area (m^{2})
    INTEGER(i_std), DIMENSION(npts,nvm),INTENT(inout)            :: last_cut           !! Years since last thinning or clearcut (years)


    !! 0.4 Local variables

    REAL(r_std), DIMENSION(ncirc,nparts)                         :: circ_class_biomass_new !! Biomass of the components of the model  
                                                                                       !! tree within a circumference
                                                                                       !! class @tex $(gC ind^{-1})$ @endtex  
    REAL(r_std), DIMENSION(ncirc)                                :: circ_class_n_new   !! Number of individuals in each circ class
                                                                                       !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                                :: old_trees_left
    REAL(r_std)                                                  :: trees_needed
    REAL(r_std)                                                  :: scale_factor
    INTEGER                                                      :: icir, jcir
    !_ ================================================================================================================================

    IF (bavard.GE.2) WRITE(numout,*) &
         'Entering second_coppice_cut sapiens_kill.f90'

    IF( SUM(circ_class_kill(ipts,ivm,:,ifm, icut))&
         .GT. min_stomate ) THEN

       IF(ld_forestry)THEN
          WRITE(numout,*) 'Doing a second coppice cut'
          WRITE(numout,*) 'ipts,ivm,ifm,icut ',ipts,ivm,ifm,icut
       ENDIF

       ! Again, we harvest all aboveground biomass and leave the roots
       CALL harvest_aboveground_wood_living_roots(npts, ipts, ivm, iele, &
            ifm, icut, biomass, bm_to_litter, &
            circ_class_biomass, circ_class_kill, harvest_pool, &
            harvest_type, harvest_cut, harvest_area, &
            harvest_pool_bound, resolution, veget_max)

       ! reset the time since the last cut
       last_cut(ipts,ivm)=0

       ! We also reset circ_class_kill for this grid square, since
       ! we clearly don't need to do mortality if we have no more
       ! aboveground biomass left.
       circ_class_kill(ipts,ivm,:,:,:)=zero

       ! Now we have a certain number density and a certain amount of
       ! biomass.  We want to have the same number of trees that we had
       ! during the first cut, since the stumps from the first part aren't
       ! going to change (maybe a couple will die, but not a large difference).

       ! This code is copied from lpj_gap.f90.
       circ_class_n_new(:)=circ_class_dist(:)*ind(ipts,ivm)
       circ_class_biomass_new(:,:)=zero

       ! now we populate the new classes
       old_trees_left(:)=circ_class_n(ipts,ivm,:)
       DO icir=1,ncirc

          trees_needed=circ_class_n_new(icir)
          DO jcir=1,ncirc

             IF(trees_needed .LE. zero)EXIT

             IF(old_trees_left(jcir) .GE. trees_needed )THEN
                ! we can get all the trees we need from this class
                ! and don't have to continue searching
                circ_class_biomass_new(icir,:)=circ_class_biomass_new(icir,:)+&
                     circ_class_biomass(ipts,ivm,jcir,:,icarbon)*trees_needed
                old_trees_left(jcir)=old_trees_left(jcir)-trees_needed
                trees_needed = zero
                EXIT
             ELSE

                ! the trees in this class are not sufficient, so we
                ! need to move all of them to the new class.
                circ_class_biomass_new(icir,:)=circ_class_biomass_new(icir,:)+&
                     circ_class_biomass(ipts,ivm,jcir,:,icarbon)*&
                     old_trees_left(jcir)
                trees_needed = trees_needed -old_trees_left(jcir)
                old_trees_left(jcir)=zero

             ENDIF ! checking if we have enough trees

          ENDDO ! jcir=1,ncirc

       ENDDO ! icir=1,ncirc

       ! right now, circ_class_biomass_new gives the total biomass
       ! in each class, but it should only be for a model tree. 
       ! So let's normalize it.
       DO icir=1,ncirc
          circ_class_biomass_new(icir,:)=circ_class_biomass_new(icir,:)&
               /circ_class_n_new(icir)
       ENDDO

       ! now update the variables that pass this information around
       DO icir=1,ncirc
          circ_class_biomass(ipts,ivm,icir,:,icarbon)=circ_class_biomass_new(icir,:)
          circ_class_n(ipts,ivm,icir)=circ_class_n_new(icir)
       ENDDO

       !----- end of copied code..make a subroutine?


       ! reset the time since the last cut
       last_cut(ipts,ivm)=0

       ! We also reset circ_class_kill for this grid square, since
       ! we clearly don't need to do mortality if we have no more
       ! aboveground biomass left.
       circ_class_kill(ipts,ivm,:,:,:)=zero

       ! After a first coppicing, we will regrow a certain number
       ! of shoots per stool.  For a second coppice cut, our starting
       ! density is exactly the same as the first cut.
       
       scale_factor=coppice_dens(ipts,ivm)/SUM(circ_class_n(ipts,ivm,:))*&
            REAL(shoots_per_stool(ivm),r_std)

       circ_class_n(ipts,ivm,:)=circ_class_n(ipts,ivm,:)*&
            scale_factor
       ind(ipts,ivm)=ind(ipts,ivm)*&
            scale_factor

       ! in order not to change the total biomass, we need to
       ! make all our model trees smaller.
       circ_class_biomass(ipts,ivm,:,:,:)=circ_class_biomass(ipts,ivm,:,:,:)/&
            scale_factor

    ENDIF ! any individuals to second cut coppice?

    IF (bavard.GE.2) WRITE(numout,*) 'Leaving second_coppice_cut sapiens_kill.f90'

  END SUBROUTINE second_coppice_cut

!! ================================================================================================================================
!! SUBROUTINE   : first_coppice_cut
!!
!>\BRIEF        Transfer dead biomass to litter and wood to harvest pool.
!!
!! DESCRIPTION  : This is a total harvest of the aboveground biomass of a
!!                stand in such a way that the belowground wood continues to
!!                live.  This results in a sprouting in the following year of
!!                several shoots per stump.  Biomass is turned to litter, harvested,
!!                and left alive, depending on the pool.  The number of individuals
!!                goes up and the size of the model tree goes down.
!!
!!                NOTE: All branches and stems are harvested.  Basically, 
!!                      this means all aboveground woody biomass.  The fine roots
!!                      are killed and left to litter.  The sapwood and heartwood
!!                      belowground is left alive.  The leaves are killed, and the
!!                      labile and carbres pools are left untouched in the living
!!                      biomass.
!!
!! RECENT CHANGE(S): 
!!
!! MAIN OUTPUT VARIABLE(S): ::bm_to_litter, ::harvest_pool
!!
!! REFERENCE(S)   :
!!
!! FLOWCHART    : None
!!\n
!_ ================================================================================================================================
 
  SUBROUTINE first_coppice_cut (npts, ipts, ivm, iele, ifm, &
       icut, harvest_pool_bound, resolution, veget_max, biomass, &
       ind, circ_class_biomass, circ_class_n, circ_class_kill, &
       bm_to_litter, harvest_pool, harvest_type, harvest_cut, &
       harvest_area, last_cut)


    !! 0. Variable and parameter declaration

    !! 0.1 Input variables 

    INTEGER(i_std), INTENT(in)                                   :: npts               !! Domain size (-)
    INTEGER(i_std), INTENT(in)                                   :: ipts               !! current grid square
    INTEGER(i_std), INTENT(in)                                   :: ivm                !! current PFT
    INTEGER(i_std), INTENT(in)                                   :: iele               !! current element type
    INTEGER(i_std), INTENT(in)                                   :: ifm                !! current fm strategy
    INTEGER(i_std), INTENT(in)                                   :: icut               !! current cut time
    REAL(r_std), DIMENSION(0:ndia_harvest+1), INTENT(in)         :: harvest_pool_bound !! The boundaries of the diameter classes
                                                                                       !! in the wood harvest pools
                                                                                       !! @tex $(m)$ @endtex
    REAL(r_std),DIMENSION(npts,2), INTENT(in)                    :: resolution         !! The length of each grid-box in X and Y
                                                                                       !! @tex $(m)$ @endtex
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)                   :: veget_max          !! Maximum fraction of vegetation type 


    !! 0.2 Output variables


    !! 0.3 Modified variables   

    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                             INTENT(inout)       :: biomass            !! Biomass @tex $(gC m^{-2}) $@endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)              :: ind                !! Stand level number of individuals 
                                                                                       !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nparts,nelements), &                         
                                              INTENT(inout)      :: circ_class_biomass !! Biomass of the components of the model  
                                                                                       !! tree within a circumference
                                                                                       !! class @tex $(gC ind^{-1})$ @endtex  
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(inout)        :: circ_class_n       !! Number of individuals in each circ class
                                                                                       !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nfm_types,ncut_times), &
                                                  INTENT(inout)  :: circ_class_kill    !! Number of trees within a circ that needs
                                                                                       !! to be killed @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                                  INTENT(inout)  :: bm_to_litter       !! Biomass transfer to litter 
                                                                                       !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)               :: harvest_pool       !! The wood and biomass that have been
                                                                                       !! havested by humans @tex $(gC)$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)                   :: harvest_type       !! Type of management that resulted
                                                                                       !! in the harvest (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)                   :: harvest_cut        !! Type of cutting that was used for the harvest
                                                                                       !! (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)                   :: harvest_area       !! Harvested area (m^{2})
    INTEGER(i_std), DIMENSION(npts,nvm),INTENT(inout)            :: last_cut           !! Years since last thinning or clearcut (years)

    !! 0.4 Local variables

    !_ ================================================================================================================================

    IF (bavard.GE.2) WRITE(numout,*) 'Entering first_coppice_cut sapiens_kill.f90'

    ! What if we coppice for the first cut?
    IF( SUM(circ_class_kill(ipts,ivm,:,ifm, icut))&
         .GT. min_stomate ) THEN

       IF(ld_forestry)THEN
          WRITE(numout,*) 'Doing a first coppice cut'
          WRITE(numout,*) 'ipts,ivm,ifm,icut ',ipts,ivm,ifm,icut
       ENDIF

       CALL harvest_aboveground_wood_living_roots(npts, ipts, ivm, iele, &
            ifm, icut, biomass, bm_to_litter, &
            circ_class_biomass, circ_class_kill, harvest_pool, &
            harvest_type, harvest_cut, harvest_area, &
            harvest_pool_bound, resolution, veget_max)

       ! reset the time since the last cut
       last_cut(ipts,ivm)=0

       ! We also reset circ_class_kill for this grid square, since
       ! we clearly don't need to do mortality if we have no more
       ! aboveground biomass left.
       circ_class_kill(ipts,ivm,:,:,:)=zero

       ! After a first coppicing, we will regrow a certain number
       ! of shoots per stool.  Therefore, our density starting
       ! next year will be much higher.
       circ_class_n(ipts,ivm,:)=circ_class_n(ipts,ivm,:)*&
            REAL(shoots_per_stool(ivm),r_std)
       ind(ipts,ivm)=ind(ipts,ivm)*&
            REAL(shoots_per_stool(ivm),r_std)

       ! in order not to change the total biomass, we need to
       ! make all our model trees smaller.
       circ_class_biomass(ipts,ivm,:,:,:)=circ_class_biomass(ipts,ivm,:,:,:)/&
            REAL(shoots_per_stool(ivm),r_std)

    ENDIF ! any trees to first cut coppice?


    IF (bavard.GE.2) WRITE(numout,*) 'Leaving first_coppice_cut sapiens_kill.f90'

  END SUBROUTINE first_coppice_cut

!! ================================================================================================================================
!! SUBROUTINE   : clearcut_harvest_aboveground_dead_roots
!!
!>\BRIEF        Transfer dead biomass to litter and wood to harvest pool 
!!              for a whole stand.
!!
!! DESCRIPTION  : Moves dead biomass to the litter pool and the harvest pool
!!                where appropriate, assuming the whole stand is harvested.  Various
!!                counters are reset.
!!
!!                NOTE: If harvest_fraction is 1, all the aboveground biomass is
!!                      harvested.  If it's 1-branch_ratio, only stems are 
!!                      harvested.  Belowground wood, roots, fruits, and
!!                      leaves all become litter.  A portion of aboveground
!!                      wood is put in the harvest pool.  Reserves and labile
!!                      carbon are assumed to be distributed evenly across
!!                      all non-heartwood components.
!!
!! RECENT CHANGE(S): 
!!
!! MAIN OUTPUT VARIABLE(S): ::bm_to_litter, ::harvest_pool
!!
!! REFERENCE(S)   :
!!
!! FLOWCHART    : None
!!\n
!_ ================================================================================================================================
 
  SUBROUTINE clearcut_harvest_aboveground_dead_roots (npts, ipts, ivm, iele, &
       ifm, icut, biomass, ind, bm_to_litter, &
       circ_class_biomass, circ_class_kill, circ_class_n, harvest_pool, &
       harvest_type, harvest_cut, harvest_area, &
       harvest_pool_bound, resolution, veget_max, last_cut, age_stand, &
       mai_count, harvest_fraction)


    !! 0. Variable and parameter declaration

    !! 0.1 Input variables 
    INTEGER(i_std), INTENT(in)                                   :: npts               !! Domain size (-)
    INTEGER(i_std), INTENT(in)                                   :: ipts               !! current grid square
    INTEGER(i_std), INTENT(in)                                   :: ivm                !! current PFT
    INTEGER(i_std), INTENT(in)                                   :: iele               !! current element type
    INTEGER(i_std), INTENT(in)                                   :: ifm                !! current fm strategy
    INTEGER(i_std), INTENT(in)                                   :: icut               !! current cut time
    REAL(r_std), DIMENSION(0:ndia_harvest+1), INTENT(in)         :: harvest_pool_bound !! The boundaries of the diameter classes
                                                                                       !! in the wood harvest pools
                                                                                       !! @tex $(m)$ @endtex
    REAL(r_std),DIMENSION(npts,2), INTENT(in)                    :: resolution         !! The length of each grid-box in X and Y
                                                                                       !! @tex $(m)$ @endtex
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)                   :: veget_max          !! Maximum fraction of vegetation type 
    REAL(r_std),INTENT(in)                                       :: harvest_fraction   !! The fraction of the aboveground
                                                                                       !! biomass that is harvested.  This is
                                                                                       !! generally 1-branch_ratio if you only
                                                                                       !! want to harvest the stems, and 1 if
                                                                                       !! you want to harvest everything.


    !! 0.2 Output variables


    !! 0.3 Modified variables   
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                             INTENT(inout)       :: biomass            !! Biomass @tex $(gC m^{-2}) $@endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)              :: ind                !! Stand level number of individuals 
                                                                                       !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nparts,nelements), &                         
                                              INTENT(inout)      :: circ_class_biomass !! Biomass of the components of the model  
                                                                                       !! tree within a circumference
                                                                                       !! class @tex $(gC ind^{-1})$ @endtex  
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(inout)        :: circ_class_n       !! Number of individuals in each circ class
                                                                                       !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nfm_types,ncut_times), &
                                                  INTENT(inout)  :: circ_class_kill    !! Number of trees within a circ that needs
                                                                                       !! to be killed @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                                  INTENT(inout)  :: bm_to_litter       !! Biomass transfer to litter 
                                                                                       !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)               :: harvest_pool       !! The wood and biomass that have been
                                                                                       !! havested by humans @tex $(gC)$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)                   :: harvest_type       !! Type of management that reulted
                                                                                       !! in the harvest (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)                   :: harvest_cut        !! Type of cutting that was used for the harvest
                                                                                       !! (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)                   :: harvest_area       !! Harvested area (m^{2})
    INTEGER(i_std), DIMENSION(npts,nvm),INTENT(inout)            :: last_cut           !! Years since last thinning or clearcut (years)
    INTEGER(i_std), DIMENSION(npts,nvm),INTENT(inout)            :: age_stand          !! Age of stand (years)
    INTEGER(i_std), DIMENSION(npts,nvm),INTENT(inout)            :: mai_count          !! The number of times we've
                                                                                       !! calculated the volume increment
                                                                                       !! for a stand
    !! 0.4 Local variables

    !_ ================================================================================================================================

    IF (bavard.GE.2) WRITE(numout,*) &
         'Entering clearcut_harvest_aboveground_dead_roots sapiens_kill.f90'

    IF( SUM(circ_class_kill(ipts,ivm,:,ifm, icut))&
         .GT. min_stomate ) THEN

       CALL harvest_aboveground_dead_roots(npts, ipts, ivm, iele, &
            ifm, icut, biomass, ind, bm_to_litter, &
            circ_class_biomass, circ_class_kill, circ_class_n, harvest_pool, &
            harvest_type, harvest_cut, harvest_area, &
            harvest_pool_bound, resolution, veget_max, &
            harvest_fraction)

       ! We have to reset the age of the stand and the time since the
       ! last human intervention.
       age_stand(ipts,ivm)=0
       last_cut(ipts,ivm)=0
       mai_count(ipts,ivm)=0

       ! We also reset circ_class_kill for this grid square, since
       ! we clearly don't need to do mortality if we have no more
       ! trees left.
       circ_class_kill(ipts,ivm,:,:,:)=zero

    ENDIF

    IF (bavard.GE.2) WRITE(numout,*) &
         'Leaving clearcut_harvest_aboveground_dead_roots sapiens_kill.f90'

  END SUBROUTINE clearcut_harvest_aboveground_dead_roots

END MODULE sapiens_kill
